/**
 * GMyth Library
 *
 * @file gmyth/gmyth_stringlist.c
 * 
 * @brief <p> This component contains functions for dealing with the stringlist
 * format of the mythprotocol.
 * 
 * Copyright (C) 2006 INdT - Instituto Nokia de Tecnologia.
 * @author Hallyson Luiz de Morais Melo <hallyson.melo@indt.org.br>
 *
 *//*
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "gmyth_stringlist.h"

static void gmyth_string_list_class_init          (GMythStringListClass *klass);
static void gmyth_string_list_init                (GMythStringList *object);

static void gmyth_string_list_dispose  (GObject *object);
static void gmyth_string_list_finalize (GObject *object);

G_DEFINE_TYPE(GMythStringList, gmyth_string_list, G_TYPE_OBJECT)
    
static void
gmyth_string_list_class_init (GMythStringListClass *klass)
{
	GObjectClass *gobject_class;

    gobject_class = (GObjectClass *) klass;
	
    gobject_class->dispose  = gmyth_string_list_dispose;
    gobject_class->finalize = gmyth_string_list_finalize;	
}

static void
gmyth_string_list_init (GMythStringList *gmyth_string_list)
{
	gmyth_string_list->glist = NULL;
}

static void
gmyth_string_list_dispose  (GObject *object)
{
	GMythStringList *gmyth_string_list = GMYTH_STRING_LIST(object);

	if (gmyth_string_list->glist)
		gmyth_string_list_clear_all(gmyth_string_list);

	G_OBJECT_CLASS (gmyth_string_list_parent_class)->dispose (object);
}

static void
gmyth_string_list_finalize (GObject *object)
{
	//GMythStringList *gmyth_string_list = GMYTH_STRING_LIST(object);

	g_signal_handlers_destroy (object);

	G_OBJECT_CLASS (gmyth_string_list_parent_class)->finalize (object);
}

/** Creates a new instance of GStringList.
 * 
 * @return a new instance of GStringList.
 */
GMythStringList *
gmyth_string_list_new ()
{
    GMythStringList *gmyth_string_list = GMYTH_STRING_LIST (g_object_new(GMYTH_STRING_LIST_TYPE, NULL));

    return gmyth_string_list;
}

/** Appends a guint64 to the string list.
 * 
 * @param strlist The GMythStringList instance.
 * @param value The guint64 to be appended.
 * 
 * @return The appended guint64 converted to a GString object.
 */
GString*
gmyth_string_list_append_int ( GMythStringList *strlist, const gint value )
{
	GString *tmp_str = g_string_new ("");

	g_string_printf (tmp_str, "%d", value);

	gmyth_string_list_append_string (strlist, tmp_str);

	return tmp_str;	
}

/** Appends a guint64 to the string list.
 * 
 * @param strlist The GMythStringList instance.
 * @param value The guint64 to be appended.
 * 
 * @return The appended guint64 converted to a GString object.
 */
GString*
gmyth_string_list_append_uint64 ( GMythStringList *strlist, const guint64 value)
{
	GString *tmp_str = g_string_new ("");	

	glong l2 = ( (guint64)(value) & 0xffffffffLL );
	glong l1 = ( ((guint64)(value) >> 32 ) & 0xffffffffLL );
  
	/* high order part of guint64 value */
	g_string_printf (tmp_str, "%ld", l1);
	
	g_debug( "[%s] uint64 (high) = %s\n", __FUNCTION__, tmp_str->str );

	gmyth_string_list_append_string (strlist, tmp_str);

 	/* low order part of guint64 value */
	g_string_printf (tmp_str, "%ld", l2);	

	g_debug( "[%s] uint64 (low) = %s\n", __FUNCTION__, tmp_str->str );

	gmyth_string_list_append_string (strlist, tmp_str);

	return tmp_str;	
}

/** Appends a char array to the string list.
 * 
 * @param strlist The GMythStringList instance.
 * @param value The char array to be appended.
 * 
 * @return The appended char array converted to a GString object.
 */
GString*
gmyth_string_list_append_char_array ( GMythStringList *strlist, const gchar* value )
{
	GString *tmp_str = NULL;
	
	g_return_val_if_fail( strlist != NULL, NULL );
	
	tmp_str = g_string_new (value);

	g_return_val_if_fail( tmp_str != NULL, NULL );

	gmyth_string_list_append_string (strlist, tmp_str);

	return tmp_str;		
}

/** Appends a string to the string list.
 * 
 * @param strlist The GMythStringList instance.
 * @param value The string to be appended.
 * 
 * @return The appended string itself. 
 */
GString*
gmyth_string_list_append_string ( GMythStringList *strlist, GString *value )
{
	g_return_val_if_fail( strlist != NULL, NULL );

	strlist->glist = g_list_append (strlist->glist, value);

	return value;
}

/** Gets an integer value from the string list at the given position.
 * 
 * @param strlist The GMythStringList instance.
 * @param index the integer position in the list, starting with zero.
 * @return The integer value.
 */
gint
gmyth_string_list_get_int ( GMythStringList *strlist, const gint index )
{
	//TODO: Create static method check_index()
	GString *tmp_str = NULL;
	
	g_return_val_if_fail( strlist != NULL, 0 );

	tmp_str = (GString *) g_list_nth_data (strlist->glist, index);

	g_return_val_if_fail( tmp_str != NULL && tmp_str->str != NULL, 0 );
	
	return (int) ( 0x00000000ffffffffL & g_ascii_strtoull ( tmp_str->str, NULL, 0 ) );
}

/** Gets a guint64 value from the string list at the given position.
 * According to the Mythtv protocol, the 64 bits value is formed by
 * two strings.
 * 
 * @param strlist The GMythStringList instance.
 * @param index the index of the first string forming the 64 bits value. 
 * Index starts with zero.
 * @return The guint64 value.
 */
guint64
gmyth_string_list_get_uint64 ( GMythStringList *strlist, const gint index )
{
	//TODO: Create static method check_index()
	guint64 ret_value = 0;

	g_return_val_if_fail( strlist != NULL, 0 );
  
	const GString *tmp_str1 = (GString *) g_list_nth_data (strlist->glist, index);
	const GString *tmp_str2 = (GString *) g_list_nth_data (strlist->glist, index+1);
	
	glong l1 = (glong)g_ascii_strtoull (tmp_str1->str, NULL, 10);
	glong l2 = (glong)g_ascii_strtoull (tmp_str2->str, NULL, 10);
	
	ret_value = ((guint64)(l2) & 0xffffffffLL) | ((guint64)(l1) << 32);
  
	g_debug( "[%s] returning uint64 value = %llu\n", __FUNCTION__, ret_value );	
	
	return ret_value;
}

/** Gets a string from the string list at the given position.
 * 
 * @param strlist The GMythStringList instance.
 * @param index the string position in the list, starting with zero.
 * @return A pointer to the string data.
 */
GString*
gmyth_string_list_get_string ( GMythStringList *strlist, const gint index )
{
	if (!strlist || !(strlist->glist)) {
		g_warning ("%s received Null arguments", __FUNCTION__);
		return NULL;
	}

	return (GString *) g_list_nth_data (strlist->glist, index);
}

/** Removes all strings from the string list.
 * 
 * @param strlist The GMythStringList instance.
 */
void
gmyth_string_list_clear_all ( GMythStringList *strlist )
{
	if ( strlist != NULL && strlist->glist ) {
		g_list_free (strlist->glist);
		strlist->glist = NULL;
	}
}

/** Retrieves the number of elements in the string list.
 * 
 * @param strlist The GMythStringList instance.
 * @return the string list length.
 */
gint
gmyth_string_list_length ( GMythStringList *strlist )
{
	g_return_val_if_fail( strlist != NULL && strlist->glist != NULL, 0 );

	return g_list_length (strlist->glist);
}
